import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

import 'store.dart';

typedef Model ModelInitializer<Model>();

class MvuProvider<Model, Action> extends StatelessWidget {
  final ModelInitializer<Model> initialize;
  final Updater<Model, Action> update;
  final Widget child;

  const MvuProvider({
    @required this.initialize,
    @required this.child,
    @required this.update,
  });

  static void dispatch(BuildContext context, dynamic action) {
    final notifier = Provider.of<Dispatcher>(
      context,
      listen: false,
    );
    notifier(action);
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<MvuStore<Model, Action>>(
      create: (context) => MvuStore<Model, Action>(
        initialState: initialize(),
        update: update,
      ),
      child: Provider<Dispatcher>(
        create: (context) => (action) =>
            context.read<MvuStore<Model, Action>>().dispatch(action),
        child: Builder(
          builder: (context) => Provider.value(
            value: context
                .select<MvuStore<Model, Action>, Model>((value) => value.state),
            child: child,
          ),
        ),
      ),
    );
  }
}

extension MvuBuildContextExtensions on BuildContext {
  void dispatch(dynamic action) => MvuProvider.dispatch(this, action);
}
