import 'package:flutter/widgets.dart';

import 'action.dart';

typedef Model Updater<Model, Action>(
  Model previousModel,
  Action action,
);

typedef void Dispatcher(dynamic action);

class MvuStore<Model, Action> extends ChangeNotifier {
  final Updater<Model, Action> update;

  MvuStore({
    @required Model initialState,
    @required this.update,
  }) : this._state = initialState;

  Model _state;
  Model get state => _state;

  Future<void> dispatch(dynamic action) async {
    if (action is AsynchronousAction<Model>) {
      await action.execute(() => _state, dispatch);
    } else if (action is Action) {
      _state = update(_state, action);
      notifyListeners();
    }
  }
}
